﻿using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.DotProlog.InputOutput;
using OpcLabs.DotProlog.Parsing;

namespace DotPrologTest.Parsing
{
    [TestClass]
    public class TokenizerTest
    {
        #region Test Methods
        // ReSharper disable InconsistentNaming

        [TestMethod]
        public void Test_AnimalFact()
        {
            var target = new Tokenizer();
            IEnumerable<Token> tokens = target.Tokenize(TextStream.Create("animal(cat)."));
            Assert.IsNotNull(tokens);
            Token[] expected = 
            {
                Token.CreateName("animal"),
                Token.Create(TokenType.OpenCt),
                Token.CreateName("cat"),
                Token.Create(TokenType.Close),
                Token.Create(TokenType.End)
            };
            Token[] tokenArray = tokens.ToArray();
            CollectionAssert.AreEqual(expected, tokenArray);
        }

        [TestMethod]
        public void Test_AppendClause1()
        {
            var target = new Tokenizer();
            IEnumerable<Token> tokens = target.Tokenize(TextStream.Create("append([],X,X)."));
            Assert.IsNotNull(tokens);
            Token[] tokenArray = tokens.ToArray();
            CollectionAssert.AreEqual(new[] 
                {
                    Token.CreateName("append"),
                    Token.Create(TokenType.OpenCt),
                    Token.Create(TokenType.OpenList),
                    Token.Create(TokenType.CloseList),
                    Token.Create(TokenType.Comma),
                    Token.CreateVariable("X"),
                    Token.Create(TokenType.Comma),
                    Token.CreateVariable("X"),
                    Token.Create(TokenType.Close),
                    Token.Create(TokenType.End)
                },
                tokenArray);
        }

        [TestMethod]
        public void Test_AppendClause2()
        {
            var target = new Tokenizer();
            IEnumerable<Token> tokens = target.Tokenize(TextStream.Create("append([T|Q1],X,[T|Q2]) :- append(Q1,X,Q2)."));
            Assert.IsNotNull(tokens);
            Token[] tokenArray = tokens.ToArray();
            CollectionAssert.AreEqual(new[] 
                {
                    Token.CreateName("append"),
                    Token.Create(TokenType.OpenCt),
                    Token.Create(TokenType.OpenList),
                    Token.CreateVariable("T"),
                    Token.Create(TokenType.HtSep),
                    Token.CreateVariable("Q1"),
                    Token.Create(TokenType.CloseList),
                    Token.Create(TokenType.Comma),
                    Token.CreateVariable("X"),
                    Token.Create(TokenType.Comma),
                    Token.Create(TokenType.OpenList),
                    Token.CreateVariable("T"),
                    Token.Create(TokenType.HtSep),
                    Token.CreateVariable("Q2"),
                    Token.Create(TokenType.CloseList),
                    Token.Create(TokenType.Close),
                    Token.CreateName(":-"),
                    Token.CreateName("append"),
                    Token.Create(TokenType.OpenCt),
                    Token.CreateVariable("Q1"),
                    Token.Create(TokenType.Comma),
                    Token.CreateVariable("X"),
                    Token.Create(TokenType.Comma),
                    Token.CreateVariable("Q2"),
                    Token.Create(TokenType.Close),
                    Token.Create(TokenType.End)
                },
                tokenArray);
        }

        [TestMethod]
        public void Test_MultipleEnumerations()
        {
            var target = new Tokenizer();
            IEnumerable<Token> tokens = target.Tokenize(TextStream.Create("animal(cat)."));
            Assert.IsNotNull(tokens);
            Token[] expected = 
            {
                Token.CreateName("animal"),
                Token.Create(TokenType.OpenCt),
                Token.CreateName("cat"),
                Token.Create(TokenType.Close),
                Token.Create(TokenType.End)
            };
            // Multiple enumerations must give the same result.
            // ReSharper disable PossibleMultipleEnumeration
            Token[] tokenArray = tokens.ToArray();
            CollectionAssert.AreEqual(expected, tokenArray);
            Token[] tokenArray2 = tokens.ToArray();
            CollectionAssert.AreEqual(expected, tokenArray2);
            // ReSharper restore PossibleMultipleEnumeration
        }

        // ReSharper restore InconsistentNaming
        #endregion
    }
}
