﻿using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.DotProlog.Terms;
using OpcLabs.DotProlog.Test;

namespace DotPrologTest.Predicates.TypeTesting
{
    [TestClass]
    // ReSharper disable InconsistentNaming
    public class Atom_1_Test : SemiDeterministicPredicateTest
    // ReSharper restore InconsistentNaming
    {
        public class ArgumentBase : SemiDeterministicPredicateTest
        {
            public ArgumentBase() : base("atom", 1)
            {
            }

            public virtual void ArgumentTest([NotNull] Term arg, bool expected)
            {
                Test(arg, expected);
            }

            // ReSharper disable InconsistentNaming

            [TestMethod]
            public void Atom_Test()
            {
                ArgumentTest(Atom.Add("a"), true);
            }

            [TestMethod]
            public void Compound_Ground_Test()
            {
                // ReSharper disable once CoVariantArrayConversion
                ArgumentTest(new Compound(Atom.Add("a"), new[] { new PInteger(1) }), false);
            }

            [TestMethod]
            public void Compound_Nonground_Test()
            {
                // ReSharper disable once CoVariantArrayConversion
                ArgumentTest(new Compound(Atom.Add("a"), new[] { new Variable() }), false);
            }

            [TestMethod]
            public void Float_Test()
            {
                ArgumentTest(new PFloat(1.0d), false);
            }

            [TestMethod]
            public void Integer_Test()
            {
                ArgumentTest(new PInteger(1), false);
            }

            //[TestMethod]
            //public void Single_Test()
            //{
            //    ArgumentTest(new Single(1.0f), false);
            //}

            [TestMethod]
            public void String_Test()
            {
                ArgumentTest(new PString("str"), false);
            }

            [TestMethod]
            public void Variable_Test()
            {
                Test(new Variable(), false);
            }

            // ReSharper restore InconsistentNaming
        }

        [TestClass]
        // ReSharper disable InconsistentNaming
        public class Atom_1_BoundVariable_Test : ArgumentBase
        // ReSharper restore InconsistentNaming
        {
            public override void ArgumentTest(Term arg, bool expected)
            {
                Term variable = new Variable();
                Assert.IsNotNull(TermUtilities.UnifyWithoutOccursCheck(variable, arg));
                Test(variable, expected);
            }
        }

        [TestClass]
        // ReSharper disable InconsistentNaming
        public class Atom_1_BoundVariable2_Test : ArgumentBase
        // ReSharper restore InconsistentNaming
        {
            public override void ArgumentTest(Term arg, bool expected)
            {
                Term variable1 = new Variable();
                Assert.IsNotNull(TermUtilities.UnifyWithoutOccursCheck(variable1, arg));
                Term variable2 = new Variable();
                Assert.IsNotNull(TermUtilities.UnifyWithoutOccursCheck(variable2, variable1));
                Test(variable2, expected);
            }
        }

        [TestClass]
        // ReSharper disable InconsistentNaming
        public class Atom_1_DirectValue_Test : ArgumentBase
        // ReSharper restore InconsistentNaming
        {
        }



        public Atom_1_Test() : base("atom", 1)
        {
        }
    }
}
