﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Reflection;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib;
using OpcLabs.BaseLib.Testing;
using OpcLabs.BaseLib.Utilities;

namespace OpcLabs.UnitTestLib.Testers
{
    public class CopyCtorTester : Tester
    {
        public CopyCtorTester() 
        {
            TestCategories.Add("Construction");
        }

        protected override void Initialize(Type typeUnderTest)
        {
            base.Initialize(typeUnderTest);

            // ReSharper disable once LoopCanBePartlyConvertedToQuery
            foreach (KeyValuePair<string, CreatorEntry<InstancesAttribute>> instanceCreator in InstanceCreators)
            {
                string instanceName = instanceCreator.Key;
                Debug.Assert(instanceName != null);
                AddTestMethod(CreateTestMethod(typeUnderTest, instanceName));
            }
        }

        [NotNull]
        private TestMethod CreateTestMethod([NotNull] Type typeUnderTest, [NotNull] string instanceName)
        {
            return new DelegatedTestMethod(
                instanceName,
                () =>
                    {
                        ConstructorInfo copyConstructor = typeUnderTest.GetConstructor(
                            BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance,
                            binder:null,
                            types:new[] { typeUnderTest },
                            modifiers:null);
                        Assert.IsNotNull(copyConstructor, "Copy constructor not found.");

                        object instance = CreateInstance(instanceName);
                        object copy = copyConstructor.Invoke(new [] { instance });
                        Assert.IsNotNull(copy);

                        Assert.IsTrue(ObjectUtilities.ValueEquals(instance, copy), 
                            "The copy is not value equal to the original instance.");
                        // Equivalence reflexivity test as a bonus:
                        Assert.IsTrue(ObjectUtilities.ValueEquals(copy, instance),
                            "The original instance is not value equal to the copy (equivalence reflexivity problem).");
                        if (!(instance is IValueEquatable))
                            Assert.AreEqual(instance.GetHashCode(), copy.GetHashCode(),
                                "The copy has a different hash code from the original instance.");
                    });
        }
    }
}
