﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Globalization;
using System.Threading;
using System.Windows.Forms;
using JetBrains.Annotations;
using OpcLabs.BaseLib.ComponentModel.Internal;
using OpcLabs.BaseLibFormsTestApp.Internal;

namespace OpcLabs.BaseLibFormsTestApp
{
    /// <tocexclude/>
    /// <summary>
    /// Represent a class for testing ObjectTestbedForm.
    /// </summary>
    [Description("Represents a class for testing ObjectTestbedForm.")]
    [TypeConverter(typeof(ExpandableObjectConverter))]
    public class TestClass
    {
        #region Public Constants

        public const bool TestPublicConstant = false;

        #endregion

        #region Public Static Fields

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2211:NonConstantFieldsShouldNotBeVisible")]
        [Description("Represents a static field.")]
        static public TestStruct TestStructStaticField;

        #endregion

        #region Public Static Properties

        [Description("Represents a static property.")]
        static public TestStruct TestStructStaticProperty { get; set; }

        // ReSharper disable UnusedAutoPropertyAccessor.Local
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode")]
        [Description("Represents a static readonly property.")]
        static public TestStruct TestStructStaticReadOnlyProperty { get; private set; }
        // ReSharper restore UnusedAutoPropertyAccessor.Local

        #endregion

        #region PublicProperties

        [Description("Represents a property.")]
        public TestStruct TestStructProperty { get; set; }

        // ReSharper disable UnusedAutoPropertyAccessor.Local
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode")]
        [Description("Represents a readonly property.")]
        public TestStruct TestStructReadOnlyProperty { get; private set; }
        // ReSharper restore UnusedAutoPropertyAccessor.Local

        public TestStruct? TestStructNullableProperty { get; set; }

        [Description("Represents a property of struct type.")]
        public Message Message { get; set; }

        [Description("Represents a nullable int property.")]
        public int? NullableInt { get; set; }

        [Description("Represents a property of IntPtr type.")]
        public IntPtr IntPtr { get; set; }

        [Description("Represents a property of IEnumerableInt.")]
        public IEnumerable<int> EnumerableInt { get; set; }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1819:PropertiesShouldNotReturnArrays")]
        [Description("Represents a property of int array type.")]
        public int[] IntArray { get; set; }

        #endregion

        #region Public Fields

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
        public TestStruct TestStructField;

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
        public TestStruct? TestStructNullableField;

        #endregion

        #region Public Methods

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1702:CompoundWordsShouldBeCasedCorrectly", MessageId = "inStruct")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1045:DoNotPassTypesByReference", MessageId = "1#")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1822:MarkMembersAsStatic")]
        [Description("Represents a method.")]
        public TestStruct GetTestStruct(TestStruct inStruct, ref TestStruct refStruct, bool returnSecondParameter)
        {
            if (returnSecondParameter) return refStruct;
            refStruct = inStruct;
            return inStruct;
        }

        static public T GetNode<T>([NotNull] T node) where T : class
        {
            return node;
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1720:IdentifiersShouldNotContainTypeNames", MessageId = "object")]
        static public T GetTypedObject<T>([NotNull] Object anyObject)
        {
            return (T)anyObject;
        }

        public void MethodA(string text)
        {
            
        }

        public int MethodB(object obj)
        {
            return 0;
        }

        #endregion

#pragma warning disable 67
        public event EventHandler<TestEventArgs> MonitoredItemChanged;
        static public event EventHandler<TestEventArgs> StaticEventHandler;
#pragma warning restore 67



    }

    /// <tocexclude/>
    public class TestEventArgs : EventArgs
    {
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1720:IdentifiersShouldNotContainTypeNames", MessageId = "int")]
        public int IntParameter { get; set; }
    }

    /// <tocexclude/>
    static public class TestClassExtensions1
    {
        static public void MethodA(this TestClass testClass, object o)
        {
            
        }

        static public int MethodB(this TestClass testClass, string text)
        {
            return 1;
        }

        static public int MethodB(this TestClass testClass, string text, int i)
        {
            return i;
        }
    }

    static public class TestClassExtensions2
    {
        static public void MethodA(this TestClass testClass, object o)
        {

        }

        static public int MethodB(this TestClass testClass, object text)
        {
            return 2;
        }
        static public int MethodB(this TestClass testClass, string text)
        {
            return 2;
        }

        static public string MethodB(this TestClass testClass, string text, int i)
        {
            return text;
        }

        static public int MethodB(this TestClass testClass, int i, string text)
        {
            return i+2;
        }
    }

    static public class TestClass2Extensions
    {
        static public int MethodB(this TestClass testClass, int i, string text)
        {
            return i + 22;
        }
        static public int MethodB(this TestClass testClass, int i, object text)
        {
            return i + 22;
        }
    }

    /// <tocexclude/>
    public class TestForm : Form
    {
        [NotNull]
        static private readonly TestForm Instance = new TestForm();
#pragma warning disable 67
        static private readonly object MonitoredItemChangedEventKey = new Object();
        public event EventHandler<TestEventArgs> MonitoredItemChanged
        {
            add => Events.AddHandler(MonitoredItemChangedEventKey, value);
            remove => Events.RemoveHandler(MonitoredItemChangedEventKey, value);
        }
        static private readonly object StaticEventHandlerEventKey = new Object();
        static public event EventHandler<TestEventArgs> StaticEventHandler
        {
            add => Instance.Events.AddHandler(StaticEventHandlerEventKey, value);
            remove => Instance.Events.RemoveHandler(StaticEventHandlerEventKey, value);
        }
#pragma warning restore 67

        private void OnMonitoredItemChanged(TestEventArgs e)
        {
            var handler = (EventHandler<TestEventArgs>)Events[MonitoredItemChangedEventKey];
            handler?.Invoke(this, e);
        }

        static private void OnStaticEventHandler(TestEventArgs e)
        {
            ((EventHandler<TestEventArgs>)Instance.Events[StaticEventHandlerEventKey])?.Invoke(Instance, e);
        }

        static private int _counterEventCycles;
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1030:UseEventsWhereAppropriate")]
        public void RaiseEvents()
        {
            var c = _counterEventCycles++;
            Text = (10 * c + 1).ToString(CultureInfo.CurrentCulture);
            OnMonitoredItemChanged(new TestEventArgs { IntParameter = 10 * c + 1 });
            Text = (10 * c + 2).ToString(CultureInfo.CurrentCulture);
            OnMonitoredItemChanged(new TestEventArgs { IntParameter = 10 * c + 2 });
            Thread.Sleep(100);
            Text = (10 * c + 3).ToString(CultureInfo.CurrentCulture);
            OnMonitoredItemChanged(new TestEventArgs { IntParameter = 10 * c + 3 });
            OnMonitoredItemChanged(new TestEventArgs { IntParameter = 10 * c + 4 });
            InvokeMethodStart(1000 + c * 100, 10 * c + 6);
            OnMonitoredItemChanged(new TestEventArgs { IntParameter = 10 * c + 5 });
        }

        static private void InvokeMethodStart(int sleep, int intParameter)
        {
            var thread = new Thread(InvokeMethod0) { IsBackground = true };
            thread.Start(new InvokeMethod0Parameters { Sleep = sleep, IntParameter = intParameter });
        }

        private class InvokeMethod0Parameters
        {
            public int Sleep;
            public int IntParameter;
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        static private void InvokeMethod0(Object prms)
        {
            var invokeMethod0Parameters = prms as InvokeMethod0Parameters;
            if (invokeMethod0Parameters == null) return;
            try
            {

                Thread.Sleep(invokeMethod0Parameters.Sleep);
                Instance.Text = (invokeMethod0Parameters.IntParameter).ToString(CultureInfo.CurrentCulture);
                OnStaticEventHandler(new TestEventArgs { IntParameter = invokeMethod0Parameters.IntParameter });
                Thread.Sleep(invokeMethod0Parameters.Sleep);
                Instance.Text = (invokeMethod0Parameters.IntParameter + 1).ToString(CultureInfo.CurrentCulture);
                OnStaticEventHandler(new TestEventArgs { IntParameter = invokeMethod0Parameters.IntParameter + 1 });
                Thread.Sleep(invokeMethod0Parameters.Sleep);
                Instance.Text = (invokeMethod0Parameters.IntParameter + 1).ToString(CultureInfo.CurrentCulture);
                OnStaticEventHandler(new TestEventArgs { IntParameter = invokeMethod0Parameters.IntParameter + 1 });
                Thread.Sleep(invokeMethod0Parameters.Sleep);
                Instance.Text = (invokeMethod0Parameters.IntParameter + 1).ToString(CultureInfo.CurrentCulture);
                OnStaticEventHandler(new TestEventArgs { IntParameter = invokeMethod0Parameters.IntParameter + 1 });
                Thread.Sleep(invokeMethod0Parameters.Sleep);
                Instance.Text = (invokeMethod0Parameters.IntParameter + 1).ToString(CultureInfo.CurrentCulture);
                OnStaticEventHandler(new TestEventArgs { IntParameter = invokeMethod0Parameters.IntParameter + 1 });
                Thread.Sleep(invokeMethod0Parameters.Sleep);
                Instance.Text = (invokeMethod0Parameters.IntParameter + 1).ToString(CultureInfo.CurrentCulture);
                OnStaticEventHandler(new TestEventArgs { IntParameter = invokeMethod0Parameters.IntParameter + 1 });
                Thread.Sleep(invokeMethod0Parameters.Sleep);
                Instance.Text = (invokeMethod0Parameters.IntParameter + 1).ToString(CultureInfo.CurrentCulture);
                OnStaticEventHandler(new TestEventArgs { IntParameter = invokeMethod0Parameters.IntParameter + 1 });
                Thread.Sleep(invokeMethod0Parameters.Sleep);
                Instance.Text = (invokeMethod0Parameters.IntParameter + 1).ToString(CultureInfo.CurrentCulture);
                OnStaticEventHandler(new TestEventArgs { IntParameter = invokeMethod0Parameters.IntParameter + 1 });
                Thread.Sleep(invokeMethod0Parameters.Sleep);
                Instance.Text = (invokeMethod0Parameters.IntParameter + 1).ToString(CultureInfo.CurrentCulture);
                OnStaticEventHandler(new TestEventArgs { IntParameter = invokeMethod0Parameters.IntParameter + 1 });
                Thread.Sleep(invokeMethod0Parameters.Sleep);
                Instance.Text = (invokeMethod0Parameters.IntParameter + 1).ToString(CultureInfo.CurrentCulture);
                OnStaticEventHandler(new TestEventArgs { IntParameter = invokeMethod0Parameters.IntParameter + 1 });
            }
            catch (Exception)
            {
                Instance.Text = (invokeMethod0Parameters.IntParameter + 2).ToString(CultureInfo.CurrentCulture);
                OnStaticEventHandler(new TestEventArgs { IntParameter = invokeMethod0Parameters.IntParameter + 2 });
            }
        }

// ReSharper disable once UnusedMember.Local
        private void TestExtensions()
        {
            var t = new TestClass();

            t.MethodA("l");
            TestClassExtensions1.MethodA(t, new object());
            t.MethodB(new object());
            t.MethodB("xxx");

            var t2 = new TestClass2();
            //t2.MethodB(5, "k");
            t2.MethodB(5, 3, 1);
        }
    }

    /// <tocexclude/>
    public class TestClass2 : TestClass
    {
        
    }

    /// <tocexclude/>
    [TypeConverter(typeof(ExpandableStructConverter))]
    public struct TestStruct
    {
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
        public int Field1;
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
        public int Field2;
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1704:IdentifiersShouldBeSpelledCorrectly", MessageId = "Const")]
        public const int Const1 = 55;

        /// <inheritdoc cref="object.ToString()"/>
        public override string ToString()
        {
            return Field1 + ", " + Field2;
            //return base.ToString();
        }

        /// <inheritdoc cref="object.Equals(object)"/>
        public override bool Equals(object obj)
        {
// ReSharper disable once RedundantComparisonWithNull
            if ((obj == null) || !(obj is TestStruct)) return false;
            var other = (TestStruct)obj;
            return (other.Field1 == Field1) && (other.Field2 == Field2);
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1704:IdentifiersShouldBeSpelledCorrectly", MessageId = "b")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1704:IdentifiersShouldBeSpelledCorrectly", MessageId = "a")]
        static public bool operator ==(TestStruct a, TestStruct b)
        {
            return a.Field1 == b.Field1 && a.Field2 == b.Field2;
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1704:IdentifiersShouldBeSpelledCorrectly", MessageId = "a")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1704:IdentifiersShouldBeSpelledCorrectly", MessageId = "b")]
        static public bool operator !=(TestStruct a, TestStruct b)
        {
            return !(a == b);
        }

        /// <inheritdoc cref="object.GetHashCode()"/>
        public override int GetHashCode()
        {
            // ReSharper disable NonReadonlyFieldInGetHashCode
            return Field1 ^ Field2;
            // ReSharper restore NonReadonlyFieldInGetHashCode
        }
    }


}

namespace OpcLabs.BaseLibFormsTestApp.Internal
{
    static public class TestClass3Extensions
    {
        static public int MethodB(this TestClass testClass, int i, string text)
        {
            return i + 22;
        }
        static public int MethodB(this TestClass testClass, int i, int j, int k)
        {
            return i + 22;
        }
    }

}
