﻿
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.Collections.Generic.Extensions;
using OpcLabs.BaseLib.Collections.Generic.Utilities;

namespace OpcLabs.BaseLibTest.Collections.Generic.Extensions
{
    // ReSharper disable once InconsistentNaming
    [TestClass]
    public partial class IEnumerableExtensionTest
    {
        [TestMethod]
        public void ElliptTest_0_0()
        {
            var result = Enumerable.Empty<object>().Ellipt(0, "ellipsis");
            Assert.IsNotNull(result);
            Assert.AreEqual(0, result.Count());
        }

        [TestMethod]
        public void ElliptTest_0_1()
        {
            var result = Enumerable.Empty<object>().Ellipt(1, "ellipsis");
            Assert.IsNotNull(result);
            Assert.AreEqual(0, result.Count());
        }

        [TestMethod]
        public void ElliptTest_1_0()
        {
            var result = EnumerableUtilities.One("first").Ellipt(0, "ellipsis");
            Assert.IsNotNull(result);
            // ReSharper disable PossibleMultipleEnumeration
            Assert.AreEqual(1, result.Count());
            Assert.AreEqual("ellipsis", result.ElementAt(0));
            // ReSharper restore PossibleMultipleEnumeration
        }

        [TestMethod]
        public void ElliptTest_1_1()
        {
            var result = EnumerableUtilities.One("first").Ellipt(1, "ellipsis");
            Assert.IsNotNull(result);
            // ReSharper disable PossibleMultipleEnumeration
            Assert.AreEqual(1, result.Count());
            Assert.AreEqual("first", result.ElementAt(0));
            // ReSharper restore PossibleMultipleEnumeration
        }

        [TestMethod]
        public void ElliptTest_1_2()
        {
            var result = EnumerableUtilities.One("first").Ellipt(2, "ellipsis");
            Assert.IsNotNull(result);
            // ReSharper disable PossibleMultipleEnumeration
            Assert.AreEqual(1, result.Count());
            Assert.AreEqual("first", result.ElementAt(0));
            // ReSharper restore PossibleMultipleEnumeration
        }

        [TestMethod]
        public void ElliptTest_2_0()
        {
            var result = (new[] { "first", "second" }).Ellipt(0, "ellipsis");
            Assert.IsNotNull(result);
            // ReSharper disable PossibleMultipleEnumeration
            Assert.AreEqual(1, result.Count());
            Assert.AreEqual("ellipsis", result.ElementAt(0));
            // ReSharper restore PossibleMultipleEnumeration
        }

        [TestMethod]
        public void ElliptTest_2_1()
        {
            var result = (new [] {"first", "second"}).Ellipt(1, "ellipsis");
            Assert.IsNotNull(result);
            // ReSharper disable PossibleMultipleEnumeration
            Assert.AreEqual(2, result.Count());
            Assert.AreEqual("first", result.ElementAt(0));
            Assert.AreEqual("ellipsis", result.ElementAt(1));
            // ReSharper restore PossibleMultipleEnumeration
        }

        [TestMethod]
        public void ElliptTest_2_2()
        {
            var result = (new[] { "first", "second" }).Ellipt(2, "ellipsis");
            Assert.IsNotNull(result);
            // ReSharper disable PossibleMultipleEnumeration
            Assert.AreEqual(2, result.Count());
            Assert.AreEqual("first", result.ElementAt(0));
            Assert.AreEqual("second", result.ElementAt(1));
            // ReSharper restore PossibleMultipleEnumeration
        }

        [TestMethod]
        public void ElliptTest_2_3()
        {
            var result = (new[] { "first", "second" }).Ellipt(3, "ellipsis");
            Assert.IsNotNull(result);
            // ReSharper disable PossibleMultipleEnumeration
            Assert.AreEqual(2, result.Count());
            Assert.AreEqual("first", result.ElementAt(0));
            Assert.AreEqual("second", result.ElementAt(1));
            // ReSharper restore PossibleMultipleEnumeration
        }

        [TestMethod]
        public void GroupChunkTest()
        {
            var result = (new[] {"c", "b", "a", "c", "c", "a", "c", "a"}).GroupChunk(s => s, 2);
            Assert.IsNotNull(result);
            IEnumerable<string>[] resultArray = result.ToArray();
            Assert.AreEqual(resultArray.Length, 2);
            Assert.IsNotNull(resultArray[0]);
            string[] chunk0 = resultArray[0].ToArray();
            CollectionAssert.AreEqual(new[] {"c", "c", "b", "a", "a"}, chunk0);
            Assert.IsNotNull(resultArray[1]);
            string[] chunk1 = resultArray[1].ToArray();
            CollectionAssert.AreEqual(new[] { "c", "c", "a" }, chunk1);
        }
    }
}
