﻿
using System;
using System.Collections;
using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib;
using OpcLabs.BaseLib.Abstract;
using OpcLabs.BaseLib.ComponentModel.Internal;
using OpcLabs.BaseLib.DataTypeModel;
using OpcLabs.BaseLib.Graphs.Internal;
using OpcLabs.UnitTestLib;

namespace OpcLabs.BaseLibTest.ComponentModel.Internal
{
    [TestClass]
    public class PropertyForestTraversableTest
    {
        [TestMethod]
        public void Test1()
        {
            var component = new Exception("Level 0", new Exception("Level 1"));
            IForestTraversable<PropertyGridTreeEntry> forestTraversable = PropertyGridTreeEntry.GetForestTraversable(component);

            IEnumerable<PropertyGridTreeEntry> enumerable = forestTraversable.ToEnumerable();
            foreach (PropertyGridTreeEntry propertyGridTreeEntry in enumerable)
            {
                // ReSharper disable once PossibleNullReferenceException
                char indicator = propertyGridTreeEntry.IsAtDefault.HasValue ? (propertyGridTreeEntry.IsAtDefault.Value ? ' ' : '*') : '?';
                AssertUtilities.WriteLine(
                    $"{new String(' ', propertyGridTreeEntry.Level * 3)}{indicator}{propertyGridTreeEntry}");
            }
        }

        [TestMethod]
        public void Test2()
        {
            var component = new [] { "First", "Second" };
            IForestTraversable<PropertyGridTreeEntry> forestTraversable = PropertyGridTreeEntry.GetForestTraversable(component);

            IEnumerable<PropertyGridTreeEntry> enumerable = forestTraversable.ToEnumerable();
            foreach (PropertyGridTreeEntry propertyGridTreeEntry in enumerable)
            {
                // ReSharper disable once PossibleNullReferenceException
                char indicator = propertyGridTreeEntry.IsAtDefault.HasValue ? (propertyGridTreeEntry.IsAtDefault.Value ? ' ' : '*') : '?';
                AssertUtilities.WriteLine(
                    $"{new String(' ', propertyGridTreeEntry.Level * 3)}{indicator}{propertyGridTreeEntry}");
            }
        }

        [TestMethod]
        public void Test3()
        {
            var fieldDataDictionary = new FieldDataDictionary
            {
                ["name1"] = new PrimitiveData("value1"),
                ["name2"] = new PrimitiveData(new [] {1,2,3})
            };
            var component = new StructuredData(fieldDataDictionary);
            IForestTraversable<PropertyGridTreeEntry> forestTraversable = PropertyGridTreeEntry.GetForestTraversable(component);

            IEnumerable<PropertyGridTreeEntry> enumerable = forestTraversable.ToEnumerable();
            foreach (PropertyGridTreeEntry propertyGridTreeEntry in enumerable)
            {
                // ReSharper disable once PossibleNullReferenceException
                char indicator = propertyGridTreeEntry.IsAtDefault.HasValue ? (propertyGridTreeEntry.IsAtDefault.Value ? ' ' : '*') : '?';
                AssertUtilities.WriteLine(
                    $"{new String(' ', propertyGridTreeEntry.Level * 3)}{indicator}{propertyGridTreeEntry}");
            }
        }

        [TestMethod]
        public void Test4()
        {
            var fieldDataDictionary = new FieldDataDictionary
            {
                ["name1"] = new PrimitiveData("value1"),
                ["name2"] = new PrimitiveData(new StructuredData(new FieldDataDictionary
                {
                    ["a"] = new PrimitiveData(1),
                    ["b"] = new PrimitiveData(2),
                    ["c"] = new PrimitiveData()
                })),
                ["name3"] = new PrimitiveData(new StructuredData(new FieldDataDictionary())),
                ["name4"] = null,
                ["name5"] = new OpaqueData(new BitArray(new byte[] {1,2,3})),
                ["name6"] = new EnumerationData(12345),
                ["name7"] = new SequenceData(new GenericData[]{new PrimitiveData(1), new PrimitiveData(2), null})
            };
            var component = new StructuredData(fieldDataDictionary);
            IForestTraversable<PropertyGridTreeEntry> forestTraversable = PropertyGridTreeEntry.GetForestTraversable(component);

            IEnumerable<PropertyGridTreeEntry> enumerable = forestTraversable.ToEnumerable();
            foreach (PropertyGridTreeEntry propertyGridTreeEntry in enumerable)
            {
                // ReSharper disable once PossibleNullReferenceException
                char indicator = propertyGridTreeEntry.IsAtDefault.HasValue ? (propertyGridTreeEntry.IsAtDefault.Value ? ' ' : '*') : '?';
                AssertUtilities.WriteLine(
                    $"{new String(' ', propertyGridTreeEntry.Level * 3)}{indicator}{propertyGridTreeEntry}");
            }
        }

        [TestMethod]
        public void Test5()
        {
            var component = new EndpointDescriptor();
            IForestTraversable<PropertyGridTreeEntry> forestTraversable = PropertyGridTreeEntry.GetForestTraversable(component);

            IEnumerable<PropertyGridTreeEntry> enumerable = forestTraversable.ToEnumerable();
            foreach (PropertyGridTreeEntry propertyGridTreeEntry in enumerable)
            {
                // ReSharper disable once PossibleNullReferenceException
                char indicator = propertyGridTreeEntry.IsAtDefault.HasValue ? (propertyGridTreeEntry.IsAtDefault.Value ? ' ' : '*') : '?';
                AssertUtilities.WriteLine(
                    $"{new String(' ', propertyGridTreeEntry.Level * 3)}{indicator}{propertyGridTreeEntry}");
            }
        }

        [TestMethod]
        public void Test6()
        {
            var component = new EndpointDescriptor("someScheme://someHost");
            IForestTraversable<PropertyGridTreeEntry> forestTraversable = PropertyGridTreeEntry.GetForestTraversable(component);

            IEnumerable<PropertyGridTreeEntry> enumerable = forestTraversable.ToEnumerable();
            foreach (PropertyGridTreeEntry propertyGridTreeEntry in enumerable)
            {
                // ReSharper disable once PossibleNullReferenceException
                char indicator = propertyGridTreeEntry.IsAtDefault.HasValue ? (propertyGridTreeEntry.IsAtDefault.Value ? ' ' : '*') : '?';
                AssertUtilities.WriteLine(
                    $"{new String(' ', propertyGridTreeEntry.Level * 3)}{indicator}{propertyGridTreeEntry}");
            }
        }

        [TestMethod]
        public void Test7()
        {
            var fieldDataDictionary = new Wrapper<Dictionary<string, object>>(new Dictionary<string, object>
            {
                ["name1"] = "value1",
                ["name2"] = new PrimitiveData(new StructuredData(new FieldDataDictionary
                {
                    ["a"] = new PrimitiveData(1),
                    ["b"] = new PrimitiveData(2)
                }))
            });
            var component = fieldDataDictionary;
            IForestTraversable<PropertyGridTreeEntry> forestTraversable = PropertyGridTreeEntry.GetForestTraversable(component);

            IEnumerable<PropertyGridTreeEntry> enumerable = forestTraversable.ToEnumerable();
            foreach (PropertyGridTreeEntry propertyGridTreeEntry in enumerable)
            {
                // ReSharper disable once PossibleNullReferenceException
                char indicator = propertyGridTreeEntry.IsAtDefault.HasValue ? (propertyGridTreeEntry.IsAtDefault.Value ? ' ' : '*') : '?';
                AssertUtilities.WriteLine(
                    $"{new String(' ', propertyGridTreeEntry.Level * 3)}{indicator}{propertyGridTreeEntry}");
            }
        }

        [TestMethod]
        public void Test8()
        {
            var component = new[,] {{ "aa", "ab" }, { "ba", "bb" }};
            IForestTraversable<PropertyGridTreeEntry> forestTraversable = PropertyGridTreeEntry.GetForestTraversable(component);

            IEnumerable<PropertyGridTreeEntry> enumerable = forestTraversable.ToEnumerable();
            foreach (PropertyGridTreeEntry propertyGridTreeEntry in enumerable)
            {
                // ReSharper disable once PossibleNullReferenceException
                char indicator = propertyGridTreeEntry.IsAtDefault.HasValue ? (propertyGridTreeEntry.IsAtDefault.Value ? ' ' : '*') : '?';
                AssertUtilities.WriteLine(
                    $"{new String(' ', propertyGridTreeEntry.Level * 3)}{indicator}{propertyGridTreeEntry}");
            }
        }

        [TestMethod]
        public void Test9()
        {
            var component = new[,,] { { { "aaa", "aab" }, { "aba", "abb" } }, { { "baa", "bab" }, { "bba", "bbb" } } };
            IForestTraversable<PropertyGridTreeEntry> forestTraversable = PropertyGridTreeEntry.GetForestTraversable(component);

            IEnumerable<PropertyGridTreeEntry> enumerable = forestTraversable.ToEnumerable();
            foreach (PropertyGridTreeEntry propertyGridTreeEntry in enumerable)
            {
                // ReSharper disable once PossibleNullReferenceException
                char indicator = propertyGridTreeEntry.IsAtDefault.HasValue ? (propertyGridTreeEntry.IsAtDefault.Value ? ' ' : '*') : '?';
                AssertUtilities.WriteLine(
                    $"{new String(' ', propertyGridTreeEntry.Level * 3)}{indicator}{propertyGridTreeEntry}");
            }
        }

    }
}
