﻿
using System;
using System.IO;
using System.Reflection;
using JetBrains.Annotations;
using Microsoft.Extensions.FileProviders;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.Extensions.FileProviders;
using OpcLabs.BaseLib.Extensions.FileProviders.Internal;
using OpcLabs.UnitTestLib;

namespace OpcLabs.BaseLibTest.Extensions.FileProviders
{
    [TestClass]
    public class PhysicalWritableFileProviderTest
        : IWritableFileProviderTest
    {
        #region Static Private Methods

        [CanBeNull]
        static private string GetPhysicalRoot()
        {
            string root = Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location);
            AssertUtilities.WriteLine($"Root: {root}");
            return root;
        }

        #endregion



        #region Protected Properties

        protected override long MaximumFileLength => long.MaxValue;

        protected override string PhysicalRoot => _lazyPhysicalRoot.Value;
        

        #endregion



        #region Protected Methods

        protected override IFileProvider CreateFileProvider(bool failing)
        {
            if (failing)
                Assert.Inconclusive("Not achievable");
            return new PhysicalFileProvider(PhysicalRoot);
        }

        protected override IFileProvider2 CreateFileProvider2(bool failing)
        {
            if (failing)
                Assert.Inconclusive("Not achievable");
            return new PhysicalFileProvider2(PhysicalRoot);
        }

        protected override IWritableFileProvider CreateWritableFileProvider(bool failing)
        {
            if (failing)
                Assert.Inconclusive("Not achievable");
            return new PhysicalWritableFileProvider(PhysicalRoot);
        }

        #endregion



        #region Private Fields

        [NotNull] 
        private readonly Lazy<string> _lazyPhysicalRoot = new Lazy<string>(GetPhysicalRoot);

        #endregion
    }
}
