﻿using System;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.EasyOpc;
using OpcLabs.EasyOpc.DataAccess;
using OpcLabs.EasyOpc.DataAccess.AddressSpace;
using OpcLabs.EasyOpc.DataAccess.Extensions;
using OpcLabs.EasyOpc.OperationModel;
using OpcLabs.EasyOpcLiveTest.DataAccess.Environment;

namespace OpcLabs.EasyOpcLiveTest.DataAccess
{
    // ReSharper disable once InconsistentNaming
    public class IEasyDAClientTest_BrowseNodes : IEasyDAClientTestBase
    {
        public IEasyDAClientTest_BrowseNodes([NotNull] Func<IEasyDAClient> createClientFunc)
            : base(createClientFunc)
        {
        }

        [TestCategory("Live")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void BrowseNodesTest_Root()
        {
            var target = CreateClient();

            DANodeElementCollection actualResultCollection;
            try
            {
                actualResultCollection = target.BrowseNodes(
                    new ServerDescriptor(OpcLabs_KitServer.ProgId),
                    DANodeDescriptor.Root,
                    DABrowseParameters.Empty);
            }
            catch (OpcException opcException)
            {
                if (!target.SupportsTechnology(OpcTechnologies.Com))
                {
                    Assert.IsInstanceOfType(opcException.InnerException, typeof(NotSupportedException));
                    return;
                }
                throw;
            }

            Assert.IsNotNull(actualResultCollection);
            int count = actualResultCollection.Count;
            Assert.IsTrue(count >= 7);

            //TODO:
        }

        [TestCategory("Live")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void BrowseNodesTest_Simulation()
        {
            var target = CreateClient();

            DANodeElementCollection actualResultCollection;
            try
            {
                actualResultCollection = target.BrowseNodes(
                    new ServerDescriptor(OpcLabs_KitServer.ProgId),
                    new DANodeDescriptor("Simulation"),
                    DABrowseParameters.Empty);
            }
            catch (OpcException opcException)
            {
                if (!target.SupportsTechnology(OpcTechnologies.Com))
                {
                    Assert.IsInstanceOfType(opcException.InnerException, typeof(NotSupportedException));
                    return;
                }
                throw;
            }

            Assert.IsNotNull(actualResultCollection);
            int count = actualResultCollection.Count;
            Assert.IsTrue(count >= 163);

            //TODO:
        }
    }
}
