﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Globalization;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.Execution;
using OpcLabs.BaseLib.Instrumentation;
using OpcLabs.EasyOpc.UA;
using OpcLabs.EasyOpc.UA.AddressSpace;
using OpcLabs.EasyOpc.UA.Discovery;
using OpcLabs.EasyOpc.UA.Engine;
using OpcLabs.EasyOpc.UA.OperationModel;
using OpcLabs.EasyOpc.UA.Toolkit.Diagnostics;
using OpcLabs.EasyOpcLiveTest.Legacy.UA.Environment;
using OpcLabs.EasyOpcLiveTest.UA.Environment;
using OpcLabs.UnitTestLib;

namespace OpcLabs.EasyOpcLiveTest.Legacy.UA
{
    /// <summary>
    ///This is a test class for EasyUAClientTest and is intended
    ///to contain all EasyUAClientTest Unit Tests
    ///</summary>
    [TestClass]
    public partial class EasyUAClientTest
    {
        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext { get; set; }

        #region Additional test attributes
        // 
        //You can use the following additional attributes as you write your tests:
        //
        //Use ClassInitialize to run code before running the first test in the class
        //[ClassInitialize()]
        //public static void MyClassInitialize(TestContext testContext)
        //{
        //}
        //
        //Use ClassCleanup to run code after all tests in a class have run
        //[ClassCleanup()]
        //public static void MyClassCleanup()
        //{
        //}
        //
        //Use TestInitialize to run code before running each test
        //[TestInitialize()]
        //public void MyTestInitialize()
        //{
        //}
        //
        //Use TestCleanup to run code after each test has run
        //[TestCleanup()]
        //public void MyTestCleanup()
        //{
        //}
        //
        #endregion



        #region Static Public Methods

        static public void IsolatedReadTest([NotNull] UAEndpointDescriptor endpointDescriptor, [NotNull] UANodeDescriptor nodeDescriptor)
        {
            var client = new EasyUAClient { Isolated = true };

            UAAttributeData attributeData = client.Read(endpointDescriptor, nodeDescriptor); // blocking read...
            Assert.IsNotNull(attributeData);
            AssertUtilities.WriteLine(attributeData.ToString());
        }

        static public void ReadTest_Parallel(int count)
        {
            var instances = new MultithreadingObservableCollection<object>();

            for (int i = 0; i < count; i++)
            {
                instances.Add(new object());
            }

            UAEndpointDescriptor endpointDescriptor = LegacyUATestEnvironment.OpcFoundation_SampleServer.EndpointDescriptors.Http;
            LegacyUATestEnvironment.Current.PrecheckEndpoint(endpointDescriptor);
            Parallel.ForEach(instances, instance => IsolatedReadTest(endpointDescriptor, "nsu=http://test.org/UA/Data/;ns=2;i=10853"));
        }

        static public void VariousMultithreadingTest([NotNull] UAEndpointDescriptor endpointDescriptor,
            [NotNull] IEnumerable<string> tags, int count)
        {
            var client = new EasyUAClient();
            // ReSharper disable PossibleMultipleEnumeration
            SpinupServer(client, endpointDescriptor, tags);
            RunSyncLocalClient(endpointDescriptor, tags, count);
            RunAsyncLocalClient(endpointDescriptor, tags, count);
            RunAsyncLocalClientThread(endpointDescriptor, tags, count); //
            RunAsyncGlobalClient(client, endpointDescriptor, tags, count);
            // ReSharper restore PossibleMultipleEnumeration
        }

        #endregion



        #region Static Private Methods

        [NotNull]
        static private EasyUAClient CreateClient_InterschaltMde()
        {
            // Instantiate the client object
            var client = new EasyUAClient
            {
                Isolated = true,
                IsolatedParameters =
                {
                    SessionParameters =
                    {
                        AlwaysPreselectEndpoint = true,
                        //CertificateAcceptancePolicy = { AcceptAnyCertificate = true, AllowUserAcceptCertificate = false },
                        EndpointSelectionPolicy =
                        {
                            AllowedMessageSecurityModes = UAMessageSecurityModes.SecurityNone,
                            EnforceSameSite = false,
                        },
                        UserIdentity =
                        {
                            UserNameTokenInfo = { UserName = "guest", Password = "isguest2014" }
                        },
                        SessionConnectTimeout = 20000 // don't work also for default parameter value
                    },
                }
            };

            return client;
        }

        static private void LocalDiscoveryElementCollectionAsserts(
            // ReSharper disable once SuggestBaseTypeForParameter
            [CanBeNull] UADiscoveryElementCollection discoveryElementCollection)
        {
            Assert.IsNotNull(discoveryElementCollection);
            Assert.IsTrue(discoveryElementCollection.Count >= 2, "Not enough application elements.");
            CollectionAssert.AllItemsAreNotNull(discoveryElementCollection);
            CollectionAssert.AllItemsAreUnique(discoveryElementCollection);

            // TODO: assert more
        }

        static private void SpinupServer([NotNull] IEasyUAClient client, [NotNull] UAEndpointDescriptor endpointDescriptor, [NotNull] IEnumerable<string> tags)
        {
            // ReSharper disable once LocalizableElement
            Console.WriteLine("SpinupServer");
            // ReSharper disable PossibleMultipleEnumeration
            MakeOpcCall(endpointDescriptor, tags, 0);
            MakeOpcCallGlobal(client, endpointDescriptor, tags, 0);
            // ReSharper restore PossibleMultipleEnumeration
        }

        static private void RunAsyncLocalClient([NotNull] UAEndpointDescriptor endpointDescriptor, IEnumerable<string> tags, int count)
        {
            // ReSharper disable once LocalizableElement
            Console.WriteLine("RunAsyncLocalClient");
            var watch = new Stopwatch();
            watch.Start();
            var tasks = new List<Task>();
            for (int i = 0; i < count; i++)
            {
                int i1 = i;
                // ReSharper disable once AssignNullToNotNullAttribute
                tasks.Add(Task.Run(() => MakeOpcCall(endpointDescriptor, tags, i1)));
                //tasks.Add(Task.Factory.StartNew(() => MakeOpcCall(endpointDescriptor, tags, i1), TaskCreationOptions.LongRunning));
                //Thread.Sleep(1000);
            }
            Task.WaitAll(tasks.ToArray());
            // ReSharper disable once LocalizableElement
            Console.WriteLine("Test RunAsyncLocalClient took {0} ms", watch.ElapsedMilliseconds);
        }

        static private void RunAsyncLocalClientThread([NotNull] UAEndpointDescriptor endpointDescriptor, IEnumerable<string> tags, int count)
        {
            // ReSharper disable once LocalizableElement
            Console.WriteLine("RunAsyncLocalClient");
            var watch = new Stopwatch();
            watch.Start();
            var threads = new List<Thread>();
            for (int i = 0; i < count; i++)
            {
                int i1 = i;
                // ReSharper disable once AssignNullToNotNullAttribute
                var thread = new Thread(() => MakeOpcCall(endpointDescriptor, tags, i1));
                thread.Start();
                threads.Add(thread);
            }
            // ReSharper disable once PossibleNullReferenceException
            threads.ForEach(thread => thread.Join());
            // ReSharper disable once LocalizableElement
            Console.WriteLine("Test RunAsyncLocalClient took {0} ms", watch.ElapsedMilliseconds);
        }

        static private void RunSyncLocalClient([NotNull] UAEndpointDescriptor endpointDescriptor, [NotNull] IEnumerable<string> tags, int count)
        {
            // ReSharper disable once LocalizableElement
            Console.WriteLine("RunSyncLocalClient");
            var watch = new Stopwatch();
            watch.Start();
            for (int i = 0; i < count; i++)
            {
                int i1 = i;
                // ReSharper disable once PossibleMultipleEnumeration
                MakeOpcCall(endpointDescriptor, tags, i1);
            }
            // ReSharper disable once LocalizableElement
            Console.WriteLine("Test RunSyncLocalClient took {0} ms", watch.ElapsedMilliseconds);
        }

        static private void RunAsyncGlobalClient([NotNull] IEasyUAClient client, [NotNull] UAEndpointDescriptor endpointDescriptor, [NotNull] IEnumerable<string> tags, int count)
        {
            // ReSharper disable once LocalizableElement
            Console.WriteLine("RunAsyncGlobalClient");
            var watch = new Stopwatch();
            watch.Start();
            var tasks = new List<Task>();
            for (int i = 0; i < count; i++)
            {
                int i1 = i;
                // ReSharper disable once AssignNullToNotNullAttribute
                tasks.Add(Task.Run(() => MakeOpcCallGlobal(client, endpointDescriptor, tags, i1)));
            }
            Task.WaitAll(tasks.ToArray());
            // ReSharper disable once LocalizableElement
            Console.WriteLine("Test RunAsyncGlobalClient took {0} ms", watch.ElapsedMilliseconds);
        }

        static private void MakeOpcCall([NotNull] UAEndpointDescriptor endpointDescriptor, [NotNull] IEnumerable<string> tags, int id)
        {
            var watch = new Stopwatch();
            watch.Start();
            // ReSharper disable once LocalizableElement
            Console.WriteLine("ID {0} started at {1}", id, System.Environment.TickCount);
            using (var opc = new EasyUAClient())
            {
                var uaReadArguments = tags
                    // ReSharper disable once AssignNullToNotNullAttribute
                    .Select(tag => new UAReadArguments(endpointDescriptor, tag))
                    .ToList();
                // ReSharper disable once LocalizableElement
                Console.WriteLine("Tags for ID {0} selected in {1}", id, watch.ElapsedMilliseconds);

                //Testing the function to make sure new ones would start before old ones would finish.
                //Thread.Sleep(new Random().Next(300, 1500));
                // Obtain attribute data. By default, the Value attributes of the nodes will be read.
                opc.ReadMultiple(uaReadArguments.ToArray());
                // ReSharper disable once LocalizableElement
                Console.WriteLine("Reads for ID {0} finished, {1} elapsed", id, watch.ElapsedMilliseconds);
            }
            // ReSharper disable once LocalizableElement
            Console.WriteLine("ID {0} ended in {1}", id, watch.ElapsedMilliseconds);
        }

        static private void MakeOpcCallGlobal([NotNull] IEasyUAClient client, [NotNull] UAEndpointDescriptor endpointDescriptor, [NotNull] IEnumerable<string> tags, int id)
        {
            var watch = new Stopwatch();
            watch.Start();
            // ReSharper disable once LocalizableElement
            Console.WriteLine("ID {0} started", id);
            var uaReadArguments = tags
                // ReSharper disable once AssignNullToNotNullAttribute
                .Select(tag => new UAReadArguments(endpointDescriptor, tag))
                .ToList();

            // Obtain attribute data. By default, the Value attributes of the nodes will be read.
            client.ReadMultiple(uaReadArguments.ToArray());
            // ReSharper disable once LocalizableElement
            Console.WriteLine("ID {0} ended in {1}", id, watch.ElapsedMilliseconds);
        }

        #endregion



        #region Test Methods
        // ReSharper disable InconsistentNaming

        [Ignore]    // test disabled because it works only once in an appdomain. need to find out how to invoke it freely.
        [TestCategory("OpcUA")]
        [TestMethod]
        public void ClientCertificateParametersTest_Custom()
        {
            using (new StaticEnvironment())
            {
                // Note that this only works once in each application domain. 

#pragma warning disable 618
                EasyUAClient.SharedParameters.EngineParameters.ApplicationParameters.ApplicationManifest.InstanceCertificateSubject = "My Certificate";
#pragma warning restore 618
                EasyUAClient.SharedParameters.EngineParameters.ApplicationParameters.ApplicationManifest.ApplicationName = "My Application";
                EasyUAClient.SharedParameters.EngineParameters.ApplicationParameters.ApplicationManifest.ApplicationUriString = "http://" + LegacyTestDefs.MachineNames.OpcUaTest + "/MyApplication";

                LogEntryEventArgs eventArgsCaught = null;

                // Dump the loggable entry containing client certificate parameters.
                EasyUAClient.LogEntry +=
                    // ReSharper disable once PossibleNullReferenceException
                    (sender, e) => { if (e.EventId == UAEngineEventIds.ClientInstanceCertificateParameters) eventArgsCaught = e; };

                // Do something - invoke an OPC read, to trigger loggable entries.
                new EasyUAClient().ReadValue(LegacyUATestEnvironment.OpcFoundation_SampleServer.EndpointDescriptors.Http,
                    "nsu=http://test.org/UA/Data/;i=10853");

                AssertUtilities.WaitingPhase(10*1000);

                Assert.IsNotNull(eventArgsCaught);
                AssertUtilities.WriteLine(eventArgsCaught);
                string message = eventArgsCaught.Message;
                Assert.IsNotNull(message);
#pragma warning disable 618
                StringAssert.Contains(message, "SubjectName: " + EasyUAClient.SharedParameters.EngineParameters.ApplicationParameters.ApplicationManifest.InstanceCertificateSubject);
#pragma warning restore 618
                StringAssert.Contains(message, "ApplicationName: " + EasyUAClient.SharedParameters.EngineParameters.ApplicationParameters.ApplicationManifest.ApplicationName);
                StringAssert.Contains(message, "ApplicationUri: " + EasyUAClient.SharedParameters.EngineParameters.ApplicationParameters.ApplicationManifest.ApplicationUriString);
            }
        }

        //[TestCategory("Live")]
        //[TestCategory("OpcUA")]
        //[TestMethod]
        //public void ConnectionTest_InterschaltMde_Failure()
        //{

        //    if (!TcpClientUtilities.CanCreate("mde.interschalt.com", 4843))
        //        Assert.Inconclusive("Failed pre-check: Socket exception.");

        //    const string strUri = "opc.tcp://mde.interschalt.com:4843";

        //    // Instantiate the client object
        //    var client = CreateClient_InterschaltMde();
        //    client.IsolatedParameters.Session.EndpointSelectionPolicy.EnforceSameSite = false;    // artifically cause a specific connection problem

        //    // Obtain data nodes under "Objects" node
        //    var uaEndpointDescriptor = new UAEndpointDescriptor(strUri);
        //    try
        //    {
        //        client.BrowseDataNodes(uaEndpointDescriptor);
        //    }
        //    catch (UAException uaException)
        //    {
        //        AssertUtilities.WriteLine(uaException);
        //        StringAssert.Contains(uaException.Message, "System.Net.Sockets.SocketException");
        //        Exception innerException = uaException.InnerException;
        //        Assert.IsNotNull(innerException);
        //        AssertUtilities.WriteLine(innerException);
        //        Assert.IsInstanceOfType(innerException, typeof(SocketException));

        //        return;
        //    }
        //    Assert.Fail();
        //}

        [TestCategory("Live")]
        [TestCategory("OpcUA")]
        [TestMethod]
        public void ConnectionTest_InterschaltMde_Success()
        {

            const string strUri = "opc.tcp://mde.interschalt.com:4843";
            var endpointDescriptor = new UAEndpointDescriptor(strUri);

            TestEnvironment.PrecheckEndpoint(endpointDescriptor);

            // Instantiate the client object
            var client = CreateClient_InterschaltMde();
            client.IsolatedParameters.SessionParameters.EndpointSelectionPolicy.EnforceSameSite = true;    // this is actually the default now

            // Obtain data nodes under "Objects" node
            UANodeElementCollection nodeElementCollection = client.BrowseDataNodes(endpointDescriptor);
            Assert.IsNotNull(nodeElementCollection);
            int nodeElementCount = nodeElementCollection.Count;
            AssertUtilities.WriteLine(nodeElementCount);
            Assert.IsTrue(nodeElementCount >= 1);
        }

        [TestCategory("OpcUA")]
        [TestCategory("Live")]
        [TestMethod]
        public void ReadTest_Parallel()
        {
            ReadTest_Parallel(5);
        }

        // intended to reproduce http://www.opclabs.com/forum/ua-reading-writing-subscriptions/2267-opcuaclient-subscription-has-stoped-after-write-to-subscribed-object-from-other-task/5547
        [TestCategory("Live")]
        [TestCategory("OpcUA")]
        [TestMethod]
        public void TasksTest()
        {
            UAEndpointDescriptor endpointDescriptor = LegacyUATestEnvironment.OpcFoundation_SampleServer.EndpointDescriptors.OpcTcp;
            TestEnvironment.PrecheckEndpoint(endpointDescriptor);

            UANodeDescriptor nodeDescriptor = OpcFoundation_UASampleServer.NodeIdExpandedTexts.Data_Dynamic_Scalar_SByteValue;

            var client1 = new EasyUAClient();
            var client2 = new EasyUAClient();
            client1.DataChangeNotification += (sender, args) =>
            {
                AssertUtilities.WriteLine(String.Format(CultureInfo.InvariantCulture, "Event at {0}:", DateTime.Now));
                // ReSharper disable once ConvertToLambdaExpression
                AssertUtilities.WriteLine(args);
                // IMPROVE: do more

                Task.Factory.StartNew(() =>
                {
                    UANodeId nodeId = OpcFoundation_UASampleServer.NodeIdExpandedTexts.Data_Static_UserScalar_Int32Value;
                    AssertUtilities.WriteLine("Before WriteValue");
                    client2.WriteValue(endpointDescriptor, nodeId, 1);
                    AssertUtilities.WriteLine("After WriteValue");
                });
            };

            AssertUtilities.Phase("Subscribing");
            int handle = client1.SubscribeDataChange(endpointDescriptor, nodeDescriptor, 1000, state: "TestState");

            AssertUtilities.WaitingPhase(30*1000);

            AssertUtilities.Phase("Unsubscribing");
            client1.UnsubscribeMonitoredItem(handle);

            AssertUtilities.Phase("Finalizing");
        }

        [Ignore]    // IMPROVE: put back; works OK but is long-running
        [TestCategory("Live")]
        [TestCategory("OpcUA")]
        [TestMethod]
        public void VariousMultithreadingTest_SampleServer_OpcTcp()
        {
            const int count = 5;
            UAEndpointDescriptor endpointDescriptor = LegacyUATestEnvironment.OpcFoundation_SampleServer.EndpointDescriptors.OpcTcp;
            var tags = new List<string>
                    {
                        OpcFoundation_UASampleServer.NodeIdExpandedTexts.Data_Dynamic_Scalar_FloatValue,
                        OpcFoundation_UASampleServer.NodeIdExpandedTexts.Data_Dynamic_Scalar_SByteValue,
                        OpcFoundation_UASampleServer.NodeIdExpandedTexts.Data_Dynamic_UserScalar_Int32Value,
                    };
            VariousMultithreadingTest(endpointDescriptor, tags, count);
        }

        // Based on issue from ColsonD, 20150216
        [Ignore]    // IMPROVE: put back; works OK but is long-running
        [TestCategory("Live")]
        [TestCategory("OpcUA")]
        [TestMethod]
        public void VariousMultithreadingTest_TopServer_OpcTcp()
        {
            const int count = 5;
            UAEndpointDescriptor endpointDescriptor = "opc.tcp://127.0.0.1:49380";
            var tags = new List<string>
                    {
                        "nsu=TOP Server;s=Mesabi.Mixer.MixerDisplayInt",
                        "nsu=TOP Server;s=Mesabi.Mixer.MixerDisplayReal",
                        "nsu=TOP Server;s=Mesabi.Mixer.MixerRecipe"
                    };
            VariousMultithreadingTest(endpointDescriptor, tags, count);
        }

        // Based on issue, and needs configuration from: http://opclabs.com/forum/ua-com-reading-writing-subscriptions/2139-decimal-problem#4995
        [Ignore]    // IMPROVE: put back, after having assured the right server configuration is loaded
        [TestCategory("Live")]
        [TestCategory("OpcUA")]
        [TestMethod]
        public void WriteDecimalStringWithCommaToSingleTest_TopServer()
        {
            UAEndpointDescriptor endpointDescriptor = "opc.tcp://127.0.0.1:49380";
            UANodeDescriptor nodeDescriptor = "nsu=TOP Server;ns=2;s=Simulation.Analog.Test_Write";

            TestEnvironment.PrecheckEndpoint(endpointDescriptor);

            var client = new EasyUAClient();
            string stringValue = "99,99";
            client.WriteValue(endpointDescriptor, nodeDescriptor, stringValue);

            object valueReadBack = client.ReadValue(endpointDescriptor, nodeDescriptor);
            Assert.IsNotNull(valueReadBack);
            AssertUtilities.WriteLine(valueReadBack.GetType());
            AssertUtilities.WriteLine(valueReadBack);

            object valueExpected = Convert.ChangeType(stringValue, typeof (Single), CultureInfo.InvariantCulture);
            Assert.AreEqual(valueExpected, valueReadBack);
        }

        // ReSharper restore InconsistentNaming
        #endregion



        #region Private Methods

        #endregion



        #region Private Properties

        [NotNull]
        private LegacyUATestEnvironment TestEnvironment => LegacyUATestEnvironment.Current;

        #endregion
    }
}
