﻿
using JetBrains.Annotations;
using Microsoft.Extensions.FileProviders;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.Extensions.FileProviders;
using OpcLabs.BaseLibTest.Extensions.FileProviders;
using OpcLabs.EasyOpc.UA;
using OpcLabs.EasyOpc.UA.Engine;
using OpcLabs.EasyOpc.UA.Extensions;
using OpcLabs.EasyOpc.UA.Extensions.FileProviders;
using OpcLabs.EasyOpc.UA.FileTransfer;

namespace OpcLabs.EasyOpcLiveTest.UA.Extensions.FileProviders
{
    [TestClass]
    public class UADirectoryFileProviderTest
        : IWritableFileProviderTest
    {
        #region Public Constants

        // Unified Automation UaNETServer
        public const string EndpointUrlString = "opc.tcp://localhost:48030";

        #endregion



        #region Static Constructor

        static UADirectoryFileProviderTest()
        {
            EasyUAClient.SharedParameters.EngineParameters.CertificateAcceptancePolicy.TrustedEndpointUrlStrings.Add(EndpointUrlString);
        }

        #endregion



        #region Public Properties

        [NotNull]
        public UANodeDescriptor DirectoryNodeDescriptor = "ns=2;s=Demo.Files.FileSystem";

        [NotNull]
        public UAEndpointDescriptor EndpointDescriptor =
            new UAEndpointDescriptor(EndpointUrlString)
            {
                EndpointSelectionPolicy = new UAEndpointSelectionPolicy { AllowedMessageSecurityModes = UAMessageSecurityModes.SecuritySignAndEncrypt }
            }.WithUserNameIdentity("root", "secret");

        [NotNull]
        public UAEndpointDescriptor InvalidEndpointDescriptor = "!test-invalid";

        #endregion



        #region Protected Properties

        protected override long MaximumFileLength => 8192;
        // Unified Automation UaNETServer/UaServerNET.exe: 8192 (observed; roughly)

        protected override string PhysicalRoot => null;

        #endregion



        #region Protected Methods

        protected override IFileProvider CreateFileProvider(bool failing) => CreateFileProvider2(failing);

        protected override IFileProvider2 CreateFileProvider2(bool failing)
        {
            var fileTransferClient = new EasyUAFileTransferClient();
            var fileProvider = new UADirectoryFileProvider(
                fileTransferClient,
                failing ? InvalidEndpointDescriptor : EndpointDescriptor,
                DirectoryNodeDescriptor);
            return fileProvider;
        }

        protected override IWritableFileProvider CreateWritableFileProvider(bool failing)
        {
            var fileTransferClient = new EasyUAFileTransferClient();
            var fileProvider = new UADirectoryWritableFileProvider(
                fileTransferClient,
                failing ? InvalidEndpointDescriptor : EndpointDescriptor,
                DirectoryNodeDescriptor);
            return fileProvider;
        }

        #endregion
    }
}
