﻿
using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.EasyOpc.UA;
using OpcLabs.EasyOpc.UA.AddressSpace.Standard;
using OpcLabs.EasyOpc.UA.OperationModel;
using OpcLabs.EasyOpcLiveTest.UA.Environment;
using OpcLabs.UnitTestLib;

namespace OpcLabs.EasyOpcLiveTest.UA
{
    public partial class EasyUAClientTest
    {
        #region Test Methods

        [DynamicData(nameof(UALiveTestEnvironment.OpcFoundation_UASampleServer_OptimisticSupportedEndpointDescriptorDataRows), typeof(UALiveTestEnvironment))]
        [Timeout(60 * 1000)]
        [TestCategory("Live")]
        [TestCategory("OpcUA")]
        [TestCategory("TimeDomain")]
        [TestMethod]
        public void PullDataChangeNotificationTest_SubscribeDataChange(UAEndpointDescriptor endpointDescriptor)
        {
            TestSection.Arrange();
            TestEnvironment.PrecheckEndpoint(endpointDescriptor);

            var client = new EasyUAClient { PullDataChangeNotificationQueueCapacity = 1000 };

            TestSection.Act();

            int handle = client.SubscribeDataChange(endpointDescriptor,
                "nsu=http://test.org/UA/Data/;i=10853", 1000);

            int count = 0;
            int startTickCount = System.Environment.TickCount;
            do
            {
                EasyUADataChangeNotificationEventArgs eventArgs = client.PullDataChangeNotification(millisecondsTimeout: 2 * 1000);
                if (eventArgs == null)
                    continue;
                AssertUtilities.WriteLine(eventArgs);
                count++;

                Assert.IsNotNull(eventArgs);
                Exception exception = eventArgs.Exception;
                Assert.IsNull(exception);
                // IMPROVE: assert more about the event
            } while (System.Environment.TickCount < startTickCount + 10 * 1000);

            client.UnsubscribeMonitoredItem(handle);

            TestSection.Assert();

            AssertUtilities.WriteLine(count);
            Assert.IsTrue(count >= 1);
        }

        [Timeout(60 * 1000)]
        [TestCategory("Live")]
        [TestCategory("OpcUA")]
        [TestCategory("TimeDomain")]
        [TestCategory("UAAlarmsAndConditions")]
        [TestMethod]
        public void PullEventNotificationTest_SubscribeEvent()
        {
            TestSection.Arrange();
            UAEndpointDescriptor endpointDescriptor = UALiveTestEnvironment.OpcFoundation_UAAlarmConditionServer.EndpointDescriptors.OpcTcp;
            TestEnvironment.PrecheckEndpoint(endpointDescriptor);

            var client = new EasyUAClient { PullEventNotificationQueueCapacity = 1000 };

            TestSection.Act();

            int handle = client.SubscribeEvent(
                endpointDescriptor,
                UAObjectIds.Server,
                1000);

            int count = 0;
            int startTickCount = System.Environment.TickCount;
            do
            {
                EasyUAEventNotificationEventArgs eventArgs = client.PullEventNotification(millisecondsTimeout: 2 * 1000);
                if (eventArgs == null)
                    continue;
                AssertUtilities.WriteLine(eventArgs);
                count++;

                Assert.IsNotNull(eventArgs);
                Exception exception = eventArgs.Exception;
                Assert.IsNull(exception);
                // IMPROVE: assert more about the event
            } while (System.Environment.TickCount < startTickCount + 30 * 1000);

            client.UnsubscribeMonitoredItem(handle);

            TestSection.Assert();

            AssertUtilities.WriteLine(count);
            Assert.IsTrue(count >= 1);
        }

        #endregion
    }
}
