﻿
using System;
using System.Diagnostics;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.Security.Cryptography.PkiCertificates;
using OpcLabs.EasyOpc.UA;
using OpcLabs.EasyOpc.UA.Administration;
using OpcLabs.EasyOpc.UA.Application;
using OpcLabs.EasyOpc.UA.Application.Extensions;
using OpcLabs.EasyOpc.UA.Extensions;
using OpcLabs.EasyOpc.UA.Gds;
using OpcLabs.UnitTestLib;

namespace OpcLabs.OpcInteroperabilityTest.UA
{
    [TestClass]
    // ReSharper disable once InconsistentNaming
    public class IopWS202104_VirtualTest
    {
        [TestMethod]
        // ReSharper disable once IdentifierTypo
        public void RegisterToGdsTest()
        {
            EasyUAClient.SharedParameters.EngineParameters.CertificateAcceptancePolicy.AcceptAnyCertificate = true;
            // Define which GDS we will work with.
            UAEndpointDescriptor gdsEndpointDescriptor =
                ((UAEndpointDescriptor)"opc.tcp://iopsurface.iop.virtual.opcfoundation.org:58810/GlobalDiscoveryServer")
                // ReSharper disable once StringLiteralTypo
                .WithUserNameIdentity("appadmin", "demo");

            // Obtain the client application service.
            var client = new EasyUAClient();
            IEasyUAClientApplication clientApplication = client.GetService<IEasyUAClientApplication>();
            Debug.Assert(clientApplication != null);

            // Display which application we are about to work with.
            AssertUtilities.WriteLine($"Application URI string: {clientApplication.GetApplicationElement().ApplicationUriString}");

            UAApplicationManifest applicationManifest = clientApplication.GetApplicationManifest();
            AssertUtilities.WriteLine($"ApplicationManifest.InstanceOwnStorePath: {applicationManifest.InstanceOwnStorePath}");

            clientApplication.RemoveInstanceCertificate();

            // Obtain a new application certificate from the certificate manager (GDS), and store it for subsequent usage.
            IPkiCertificate certificate = clientApplication.ObtainNewCertificate(gdsEndpointDescriptor,
                new Progress<string>(s => Console.WriteLine("Progress: {0}", s)));

            // Display results
            AssertUtilities.WriteLine($"Certificate: {certificate}");

            AssertUtilities.WriteLine("Refreshing trust lists...");
            UATrustListMasks trustListMasks = clientApplication.RefreshTrustLists(gdsEndpointDescriptor);
            AssertUtilities.WriteLine($"Trust lists masks: {trustListMasks}");
        }
    }
}
